/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 *
 *  Created on: Jan 10, 2011
 *      Author: Martin Schreiber <martin.schreiber@in.tum.de>
 */

#ifndef CTRAVERSATOR_VERTEXCOORDS_PARALLEL_HPP_
#define CTRAVERSATOR_VERTEXCOORDS_PARALLEL_HPP_

#include "libsierpi/triangle/CTriangle_Factory.hpp"
#include "libsierpi/stacks/CFBStacks.hpp"

namespace sierpi
{
namespace travs
{

template <class CKernelClass, typename t_CSimulationStacksAndTypes>
class CTraversator_VertexCoords
{
public:
	/**
	 * implementation of kernel class
	 */
	CKernelClass cKernelClass;

	/**
	 * a few typedefs which could be utilized by the traversator to get more informations
	 */
	typedef typename CKernelClass::TVertexScalar TVertexScalar;
	typedef CTraversator_VertexCoords TThisClass;

	CStackReaderTopDown<char> structure_lifo_in;

	/**
	 * include automagically generated code
	 */
#include "auto/CTraversator_VertexCoords_autogenerated.hpp"
#include "../CTraversatorClassInc_SetupSinglePass.hpp"



	void action(
			t_CSimulationStacksAndTypes *cSimulationStacks
	)
	{
		structure_lifo_in.setup(cSimulationStacks->structure_stacks.forward);

		cKernelClass.traversal_pre_hook();

		(this->*sfcRecursiveMethod_Forward)(
				cTriangleFactory.vertices[0][0], cTriangleFactory.vertices[0][1],
				cTriangleFactory.vertices[1][0], cTriangleFactory.vertices[1][1],
				cTriangleFactory.vertices[2][0], cTriangleFactory.vertices[2][1]
			);

		cKernelClass.traversal_post_hook();
	}
};

}
}

#endif /* CSTRUCTURE_VERTEXDATA_H_ */
