/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 *  Created on: May 27, 2012
 *      Author: Martin Schreiber <martin.schreiber@in.tum.de>
 */

#ifndef CSIMULATION_HYPERBOLIC_SCALABILITY_GRAPH_HPP_
#define CSIMULATION_HYPERBOLIC_SCALABILITY_GRAPH_HPP_


class CSimulationHyperbolic_ScalabilityGraph
{
	/**
	 * setup invasive constraint and hint datasets.
	 *
	 * \return identifier for scalability graph
	 */
public:
	static int getInvadeHintsAndConstraints(
			int i_workload,					///< workload of simulation (number of triangles for tsunami simulation)
			int i_scalability_graph_id,		///< id of scalability graph
			int *o_max_threads,				///< maximum number of threads
			int *o_scalability_graph_size,	///< size of scalability graph
			float **o_scalability_graph,	///< scalability graph
			const char **o_information_string
	)
	{
		/*
		 * maximum number of cores
		 */
		int max_cores;


		/*
		 * maximum number of threads for given number of grid-cells
		 *
		 * more than the specified number of threads is not expected to increase the performance anymore and sometimes decreases the throughput
		 */
		int *max_threads;


		/*
		 * grid-cells for each scalability graph line
		 */
		int *scalability_sampling;


		/*
		 * number of samples
		 */
		int number_of_samples;


		/*
		 * scalability graphs
		 */
		float *scalability;

		*o_information_string = nullptr;

		switch(i_scalability_graph_id)
		{
		case 0:
		{
			*o_information_string = "\
/**\
 * SIERPI VERSION: 2012_05_24\
 *\
 * INFO: This scalability graphs were created using the following simulation configuration:\
 *\
 * AFFINITY: threads to use: 40\
	AFFINITY: max cores: 40\
	AFFINITY: setting affinities with distance of 1: [ 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39  ]\
	 + Flux solver: local lax friedrich\
	 + Bathymetry kernels: 0\
	 + CFL: 0.15\
	 + RK ORDER: 1\
	 + Order of basis functions: 1\
	 + Adaptive conforming cluster skipping:  [DEACTIVATED]\
	 + Number of threads to use: 40\
	 + Output verbose information each #nth timestep: 100\
	 + Output verbose information each n computation seconds: 1\
	 +\
	 + InitialDepth: 4\
	 + MinDepth: 4\
	 + MaxDepth: 18\
	 + WorldSceneId: 3\
	 + Cluster size when to request split: 8192\
	 + Cluster size when to request join (both children have to request a join): 4096\
	 + Initial number of triangles before setting up column: 32\
	 *\
 */\
";

			static int p_max_cores = 40;
			max_cores = p_max_cores;

//			static int p_max_threads[] = {2, 2, 2, 2, 2, 2, 2, 4, 6, 8, 5, 6, 6, 20, 20, 20, 10, 10, 14, 34, 31, 31, 30, 31, 31, 31, 31, 32, 30, 29, 30, 36, 36, 32, 36, 32, 32, 32, 32, 32, 32, 38, 38, 36, 32, 32};
			static int p_max_threads[] = {10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 20, 20, 20, 20, 20, 20, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40};
			max_threads = p_max_threads;

			static int p_scalability_sampling[] = {50, 401, 1247, 2668, 4663, 7321, 10539, 14365, 18809, 23781, 29323, 35071, 41057, 47199, 53474, 59941, 66695, 73614, 80784, 88295, 96028, 104696, 114489, 125900, 140109, 158541, 178746, 199127, 220417, 243278, 270494, 296294, 319918, 341114, 359933, 375430, 386039, 394255, 400840, 405196, 406777, 412595, 419477, 424205, 426998, 427571};
			scalability_sampling = p_scalability_sampling;

			static int p_number_of_samples = sizeof(p_scalability_sampling)/sizeof(p_scalability_sampling[0]);
			number_of_samples = p_number_of_samples;

			static float p_scalability[] = {
				2.11400,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,
				2.39603,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,
				2.58243,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,
				2.69157,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,
				2.75557,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,
				2.73476,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,
				2.76417,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,
				2.79639,	5.28634,	5.40724,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,
				2.81336,	5.27060,	6.58333,	7.50315,	8.46991,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,	9.94162,
				2.82954,	5.22526,	6.39534,	7.86780,	8.20726,	9.25327,	9.25327,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,	9.70871,
				2.83675,	5.27635,	6.42551,	7.01229,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,
				2.84878,	5.29428,	6.97075,	7.71119,	9.01134,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,
				2.84860,	5.33013,	7.01846,	8.25573,	9.58839,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.6869,
				2.84563,	5.37256,	6.66615,	8.45405,	9.35485,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.45200,	10.4520,
				2.84878,	5.48618,	6.54265,	8.78988,	9.27752,	10.04890,	10.26650,	10.26650,	10.26650,	10.26650,	10.29850,	10.29850,	10.29850,	10.29850,	10.29850,	10.31140,	10.31140,	10.31140,	10.47980,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.58920,	10.5892,
				2.85581,	5.50532,	7.47997,	9.29961,	11.16670,	12.34730,	12.98080,	13.58880,	14.06530,	14.53530,	14.53530,	14.53530,	14.53530,	14.53530,	14.53530,	14.53530,	14.53530,	14.53530,	14.53530,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.79730,	14.7973,
				2.85567,	5.32755,	7.47929,	9.43007,	11.76980,	13.08900,	14.36210,	14.45110,	15.71200,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.6585,
				2.85485,	5.31143,	7.41944,	9.19717,	11.14580,	11.81630,	12.94290,	13.00010,	13.55730,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.5036,
				2.85647,	5.33189,	7.41396,	9.17050,	10.82620,	11.51830,	12.20730,	13.02210,	13.02210,	13.60380,	14.10850,	14.10850,	14.10850,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.2289,
				2.85499,	5.40633,	7.46204,	9.26689,	11.01710,	12.11480,	13.48870,	14.06150,	14.39620,	14.94950,	15.24800,	15.24800,	15.24800,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.56870,	15.60890,	15.60890,	15.60890,	15.60890,	15.60890,	15.60890,	15.60890,	15.60890,	15.60890,	15.75720,	15.75720,	15.75720,	15.75720,	15.77070,	15.77070,	15.77070,	15.77070,	15.77070,	15.77070,	15.7707,
				2.85320,	5.53915,	7.63117,	9.51941,	11.19880,	12.83760,	14.36270,	15.12960,	15.56980,	16.23550,	16.67480,	16.67480,	16.67480,	16.67480,	16.67480,	16.67480,	16.67480,	16.67480,	16.67480,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	17.24870,	17.24870,	17.24870,	17.24870,	17.24870,	17.24870,	17.24870,	17.24870,	17.24870,	17.2487,
				2.84635,	5.47910,	7.69401,	9.59982,	11.49090,	13.28030,	15.14780,	16.39270,	16.91820,	17.81720,	18.53890,	19.18790,	19.18790,	19.21330,	19.21330,	19.21330,	19.27780,	19.27780,	19.27780,	19.52620,	19.90790,	19.90790,	19.90790,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.16110,	20.16110,	20.16110,	20.16110,	20.16110,	20.16110,	20.16110,	20.16110,	20.16110,	20.1611,
				2.84764,	5.39857,	7.68025,	9.70261,	11.54610,	13.46480,	15.33530,	16.84530,	17.10250,	18.64980,	19.31300,	19.83140,	19.83140,	19.87300,	19.87300,	19.87300,	19.87300,	19.87300,	19.87300,	19.87300,	19.87300,	19.87300,	19.96900,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.31260,	21.01470,	21.01470,	21.01470,	21.01470,	21.01470,	21.01470,	21.01470,	21.01470,	21.01470,	21.01470,	21.0147,
				2.84028,	5.52519,	7.64074,	9.72899,	11.54500,	13.60860,	15.14470,	16.75830,	17.21750,	18.52000,	19.09780,	19.62040,	19.80720,	20.00920,	20.00920,	20.00920,	20.00920,	20.00920,	20.00920,	20.00920,	20.03150,	20.03150,	20.03150,	20.98970,	21.17830,	21.17830,	21.17830,	21.17830,	21.60570,	21.60570,	22.61510,	22.61510,	22.61510,	22.61510,	22.61510,	22.61510,	22.61510,	22.61510,	22.61510,	22.6151,
				2.82285,	5.48680,	7.58555,	9.76152,	11.73100,	13.75060,	15.50410,	16.66540,	17.29900,	18.39570,	19.31650,	19.74270,	19.79470,	20.12800,	20.12800,	20.12800,	20.12800,	20.12800,	20.12800,	20.12800,	20.28340,	20.28340,	20.52860,	20.67450,	21.32500,	21.32500,	21.32500,	21.32500,	21.83140,	22.19530,	23.00310,	23.00310,	23.00310,	23.00310,	23.00310,	23.00310,	23.00310,	23.00310,	23.00310,	23.0031,
				2.78201,	5.36975,	7.56839,	9.73333,	11.68520,	13.83650,	15.26700,	16.64580,	17.32490,	18.48900,	19.23560,	19.88540,	20.12320,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.65460,	21.93910,	21.93910,	21.93910,	21.93910,	23.40870,	23.40870,	24.20520,	24.20520,	24.20520,	24.20520,	24.20520,	24.20520,	24.20520,	24.20520,	24.20520,	24.2052,
				2.73957,	5.26763,	7.62052,	9.94440,	11.97220,	14.17170,	15.49430,	17.58110,	18.09340,	19.59000,	20.04480,	20.74740,	21.24590,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.82940,	23.82940,	23.82940,	23.82940,	25.87680,	25.94010,	26.63660,	26.63660,	26.63660,	26.63660,	26.63660,	26.63660,	26.63660,	26.63660,	26.63660,	26.6366,
				2.76374,	5.50874,	7.86598,	10.12200,	12.39830,	14.76540,	16.26470,	18.31790,	18.85830,	20.68100,	21.28310,	21.92910,	22.75030,	24.65400,	24.65400,	24.65400,	24.65400,	24.65400,	24.97230,	25.41110,	25.41110,	25.41110,	25.83690,	26.17140,	26.43060,	26.43060,	26.43060,	26.43060,	29.57630,	29.57630,	29.57630,	29.70730,	29.70730,	29.70730,	29.70730,	29.70730,	29.70730,	29.70730,	29.70730,	29.7073,
				2.73857,	5.53278,	7.82749,	10.11770,	12.45210,	14.67810,	16.34550,	18.31450,	19.41190,	20.56200,	21.83560,	22.37970,	23.21470,	25.33840,	25.33840,	25.33840,	25.38590,	25.51940,	26.29050,	26.70520,	26.79390,	26.79390,	26.93690,	27.21650,	27.58420,	27.58420,	27.58420,	27.86970,	31.29420,	31.44730,	31.44730,	31.44730,	31.44730,	31.44730,	31.44730,	31.44730,	31.44730,	31.44730,	31.44730,	31.4473,
				2.71728,	5.53639,	7.80047,	10.07880,	12.34540,	14.48590,	15.96670,	18.03140,	19.22970,	19.70240,	21.54200,	21.99900,	23.42810,	25.34700,	25.34700,	25.34700,	25.71630,	25.78690,	26.33650,	27.19930,	27.45340,	27.45340,	27.45340,	28.37170,	28.47290,	28.47290,	28.47290,	28.94590,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.3354,
				2.69146,	5.50420,	7.72525,	10.04100,	12.28340,	14.27380,	15.55270,	17.62810,	18.70030,	19.31890,	20.82770,	21.58630,	23.53060,	25.29430,	25.29430,	25.49960,	26.21020,	26.51030,	26.94950,	28.00830,	28.31630,	28.68230,	28.77790,	29.15320,	29.22990,	29.22990,	29.71310,	29.88010,	33.23550,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.5884,
				2.66909,	5.45501,	7.72989,	10.10550,	12.40560,	14.29100,	15.51860,	17.48880,	18.75990,	19.21170,	20.73020,	21.49570,	23.67750,	24.81080,	24.81080,	26.02820,	26.69510,	27.07780,	27.91080,	29.05720,	29.13940,	29.51640,	29.98510,	30.48690,	30.48690,	30.48690,	30.48690,	31.23640,	34.25240,	34.62590,	34.62590,	34.76680,	34.76680,	34.76680,	34.76680,	34.84460,	34.84460,	34.84460,	34.84460,	34.8446,
				2.65678,	5.49741,	7.75469,	10.14040,	12.45360,	14.37580,	15.64190,	17.51650,	18.84550,	19.61840,	20.95810,	21.83170,	24.11320,	25.18430,	25.18430,	27.02270,	27.33340,	27.56140,	28.26660,	29.55880,	29.85280,	30.17720,	30.17720,	31.01730,	31.37020,	31.68550,	31.70850,	32.30740,	34.97740,	35.78150,	35.78150,	36.23470,	36.23470,	36.23470,	36.23470,	36.44080,	36.44080,	36.44080,	36.44080,	36.4408,
				2.64136,	5.42918,	7.70302,	10.16540,	12.41790,	14.31000,	15.91750,	17.44340,	18.87210,	19.69710,	21.06050,	21.87970,	24.16700,	25.06150,	25.38600,	27.40690,	27.92100,	27.92100,	28.47720,	29.70410,	30.06150,	30.78660,	30.78660,	31.47220,	31.60200,	32.01270,	32.12120,	33.02160,	34.85200,	36.08450,	36.26260,	37.14470,	37.14470,	37.14470,	37.14470,	37.14470,	37.14470,	37.14470,	37.14470,	37.1447,
				2.63477,	5.42941,	7.68203,	10.05500,	12.22870,	14.11100,	15.69930,	17.12310,	18.52280,	19.40730,	20.68770,	21.51990,	23.84640,	24.87540,	25.57150,	26.83110,	27.54660,	27.60460,	28.38690,	29.40740,	29.91030,	32.29570,	32.29570,	32.29570,	32.29570,	32.29570,	32.56870,	33.35030,	35.18990,	36.78170,	36.78170,	37.18690,	37.18690,	37.18690,	37.21540,	37.52960,	37.52960,	37.52960,	37.52960,	37.5296,
				2.64571,	5.44961,	7.72084,	10.06230,	12.25070,	14.17540,	15.70700,	17.38740,	18.52660,	19.44550,	20.86460,	21.68430,	23.85570,	25.13110,	25.98030,	26.99030,	27.76010,	27.76010,	28.75390,	29.84770,	30.38340,	32.49150,	32.49150,	32.49150,	32.49150,	32.81810,	33.11800,	34.01160,	35.83660,	36.80480,	36.80480,	38.29280,	38.29280,	38.29280,	38.29280,	38.29280,	38.29280,	38.29280,	38.29280,	38.2928,
				2.63680,	5.41672,	7.75834,	10.05720,	12.19820,	14.07440,	15.58850,	17.29300,	18.42130,	19.35530,	20.66180,	21.59150,	23.58080,	25.32710,	25.83510,	27.10120,	27.94960,	27.94960,	29.04780,	29.63570,	30.53490,	32.43790,	32.43790,	32.43790,	32.43790,	33.19530,	33.19530,	34.08990,	35.64940,	37.36640,	37.64220,	38.03130,	38.03130,	38.03130,	38.03130,	38.03130,	38.03130,	38.03130,	38.03130,	38.0313,
				2.62074,	5.37061,	7.61082,	10.01700,	12.19290,	14.02450,	15.52870,	17.25480,	18.39720,	19.19330,	20.60830,	21.58300,	23.50890,	25.34460,	25.85770,	26.86090,	28.14480,	28.14480,	29.10920,	29.88740,	30.60020,	32.43430,	32.43430,	32.43430,	32.43430,	33.43690,	33.43690,	34.53500,	36.24360,	38.39850,	38.39850,	39.37050,	39.37050,	39.37050,	39.37050,	39.37050,	39.37050,	39.37050,	39.37050,	39.3705,
				2.61108,	5.39319,	7.60319,	9.99883,	12.18450,	14.01890,	15.47240,	17.18100,	18.10880,	19.39710,	21.01110,	21.59570,	23.52440,	25.44270,	26.21700,	26.92730,	28.32550,	28.32550,	29.34320,	30.15100,	31.12170,	31.61990,	31.91330,	33.02770,	33.02770,	33.81010,	33.99720,	35.28800,	37.33840,	39.41970,	39.41970,	40.72520,	40.72520,	40.72520,	40.72520,	40.72520,	40.72520,	40.72520,	40.72520,	40.7252,
				2.61022,	5.37212,	7.59581,	10.01390,	12.19110,	14.05610,	15.43320,	17.16230,	17.92840,	19.23900,	20.85580,	21.42970,	23.49220,	25.19540,	25.75660,	26.73910,	28.95660,	28.95660,	28.95660,	29.70890,	30.67420,	30.88080,	31.74070,	32.48980,	32.86970,	33.33710,	33.76980,	34.76930,	35.05640,	39.29300,	39.29300,	39.93900,	39.93900,	39.93900,	39.93900,	39.93900,	39.93900,	39.93900,	39.93900,	39.9390,
				2.60127,	5.33303,	7.56550,	9.96904,	12.16590,	13.99910,	15.42000,	17.06260,	18.13440,	19.11820,	20.69500,	21.65940,	23.54980,	24.93500,	25.56520,	26.73170,	29.40470,	29.40470,	29.40470,	29.41920,	30.43270,	30.62380,	31.89190,	32.31970,	32.36800,	33.19240,	33.56520,	34.56010,	36.34720,	39.12000,	39.12000,	39.61120,	39.61120,	39.61120,	39.61120,	39.61120,	39.61120,	39.61120,	39.61120,	39.6112,
				2.58574,	5.29717,	7.52973,	9.86195,	12.25220,	14.25110,	15.45160,	17.38340,	18.18370,	19.41680,	20.23110,	21.81970,	24.14220,	25.44570,	25.53990,	27.45650,	28.17970,	28.17970,	28.75550,	31.68510,	31.68510,	31.68510,	32.31810,	33.14210,	33.14210,	34.37450,	34.37450,	34.94050,	37.01220,	39.10010,	39.10010,	40.31340,	40.31340,	40.31340,	40.31340,	40.31340,	40.31340,	40.79970,	40.79970,	40.7997,
				2.59258,	5.19308,	7.56449,	9.93554,	12.31380,	14.33570,	15.60750,	17.57390,	18.25680,	19.65730,	20.47510,	22.14340,	24.51060,	25.89780,	26.05730,	27.71930,	28.61170,	28.61170,	29.30690,	32.27550,	32.27550,	32.27550,	32.90240,	33.84860,	33.84860,	35.15910,	35.15910,	36.18470,	38.08890,	39.84810,	39.84810,	41.07370,	41.07370,	41.07370,	41.07370,	41.07370,	41.07370,	41.86980,	41.86980,	41.8698,
				2.59514,	5.13034,	7.57239,	9.93247,	12.34370,	14.37590,	15.59900,	17.56710,	18.37360,	19.69970,	20.51290,	22.24140,	24.42970,	26.04560,	26.61050,	28.00930,	28.99320,	29.02160,	29.79500,	31.63810,	32.16730,	32.17630,	33.65530,	34.03440,	34.03440,	35.18290,	35.18290,	36.92060,	38.42880,	40.28740,	40.28740,	41.50050,	41.50050,	41.50050,	41.50050,	41.90830,	41.90830,	41.90830,	41.90830,	41.9083,
				2.60043,	5.12232,	7.58589,	9.93001,	12.35110,	14.28090,	15.50890,	17.54300,	18.24720,	19.55470,	20.34020,	22.29820,	24.06720,	25.63080,	26.18510,	27.85570,	28.46720,	29.12780,	29.25130,	30.55590,	31.53010,	31.53010,	33.08920,	33.39570,	33.67080,	34.56150,	34.56150,	36.11270,	37.17770,	39.89760,	39.89760,	40.82670,	40.82670,	40.82670,	40.82670,	40.82670,	40.82670,	40.82670,	40.82670,	40.8267,
				2.57907,	5.09275,	7.52931,	9.86345,	12.18030,	14.22440,	15.29130,	17.36450,	18.14590,	19.48050,	20.25060,	21.69210,	24.05780,	25.43970,	26.19690,	27.71780,	28.38880,	28.73540,	29.24870,	30.79440,	31.23470,	31.23470,	33.17840,	33.70880,	33.78810,	34.71070,	34.71070,	35.69640,	37.87390,	39.61100,	39.61100,	41.11340,	41.11340,	41.11340,	41.11340,	41.11340,	41.11340,	41.11340,	41.11340,	41.1134
			};
			scalability = p_scalability;

			assert(sizeof(p_scalability) / sizeof(p_scalability[0]) == max_cores * number_of_samples);
		}
			break;


		default:
			std::cerr << "WARNING: SCALABILITY GRAPH NOT SPECIFIED OR INVALID! USING `1`!" << std::endl;
			// NO BREAK HERE!

		case 1:
		{
			*o_information_string = "\
/**\
 * !!!!!!!!!!!!!!!! SMOOTHED UP VERSION !!!!!!!!!!!!!!!!!\
 *\
 * SIERPI VERSION: 2012_05_24\
 *\
 * INFO: This scalability graphs were created using the following simulation configuration:\
 *\
 * AFFINITY: threads to use: 40\
	AFFINITY: max cores: 40\
	AFFINITY: setting affinities with distance of 1: [ 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39  ]\
	 + Flux solver: local lax friedrich\
	 + Bathymetry kernels: 0\
	 + CFL: 0.15\
	 + RK ORDER: 1\
	 + Order of basis functions: 1\
	 + Adaptive conforming cluster skipping:  [DEACTIVATED]\
	 + Number of threads to use: 40\
	 + Output verbose information each #nth timestep: 100\
	 + Output verbose information each n computation seconds: 1\
	 +\
	 + InitialDepth: 4\
	 + MinDepth: 4\
	 + MaxDepth: 18\
	 + WorldSceneId: 3\
	 + Cluster size when to request split: 8192\
	 + Cluster size when to request join (both children have to request a join): 4096\
	 + Initial number of triangles before setting up column: 32\
	 *\
 */\
";

			static int p_max_cores = 40;
			max_cores = p_max_cores;

//			static int p_max_threads[] = {2, 2, 2, 2, 2, 2, 2, 4, 4, 5, 5, 6, 6, 8, 9, 9, 10, 10, 14, 18, 22, 24, 25, 26, 27, 27, 28, 28, 29, 29, 30, 31, 31, 31, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32};
			static int p_max_threads[] = {10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 20, 20, 20, 20, 20, 20, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40, 40};
			max_threads = p_max_threads;

			static int p_scalability_sampling[] = {50, 401, 1247, 2668, 4663, 7321, 10539, 14365, 18809, 23781, 29323, 35071, 41057, 47199, 53474, 59941, 66695, 73614, 80784, 88295, 96028, 104696, 114489, 125900, 140109, 158541, 178746, 199127, 220417, 243278, 270494, 296294, 319918, 341114, 359933, 375430, 386039, 394255, 400840, 405196, 406777, 412595, 419477, 424205, 426998, 427571};
			scalability_sampling = p_scalability_sampling;

			static int p_number_of_samples = sizeof(p_scalability_sampling)/sizeof(p_scalability_sampling[0]);
			number_of_samples = p_number_of_samples;

			static float p_scalability[] = {
				2.11400,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,	2.36160,
				2.39603,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,	3.82850,
				2.58243,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,	4.57855,
				2.69157,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,	4.93271,
				2.75557,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,	5.11013,
				2.73476,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,	5.00523,
				2.76417,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,	5.16561,
				2.79639,	5.28634,	5.40724,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,	5.75588,
				2.81336,	5.27060,	6.58333,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,	7.50315,
				2.82954,	5.22526,	6.39534,	7.86780,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,	8.20726,
				2.83675,	5.27635,	6.42551,	7.01229,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,	8.26557,
				2.84878,	5.29428,	6.97075,	7.71119,	9.01134,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,	9.67817,
				2.84860,	5.33013,	7.01846,	8.25573,	9.58839,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.68690,	10.6869,
				2.84563,	5.37256,	6.66615,	8.45405,	9.35485,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.28210,	10.2821,
				2.84878,	5.48618,	6.54265,	8.78988,	9.27752,	10.04890,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.26650,	10.2665,
				2.85581,	5.50532,	7.47997,	9.29961,	11.16670,	12.34730,	12.98080,	13.58880,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.06530,	14.0653,
				2.85567,	5.32755,	7.47929,	9.43007,	11.76980,	13.08900,	14.36210,	14.45110,	15.71200,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.65850,	17.6585,
				2.85485,	5.31143,	7.41944,	9.19717,	11.14580,	11.81630,	12.94290,	13.00010,	13.55730,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.50360,	14.5036,
				2.85647,	5.33189,	7.41396,	9.17050,	10.82620,	11.51830,	12.20730,	13.02210,	13.02210,	13.60380,	14.10850,	14.10850,	14.10850,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.22890,	14.2289,
				2.85499,	5.40633,	7.46204,	9.26689,	11.01710,	12.11480,	13.48870,	14.06150,	14.39620,	14.94950,	15.24800,	15.24800,	15.24800,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.51810,	15.5181,
				2.85320,	5.53915,	7.63117,	9.51941,	11.19880,	12.83760,	14.36270,	15.12960,	15.56980,	16.23550,	16.67480,	16.67480,	16.67480,	16.67480,	16.67480,	16.67480,	16.67480,	16.67480,	16.67480,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.94190,	16.9419,
				2.84635,	5.47910,	7.69401,	9.59982,	11.49090,	13.28030,	15.14780,	16.39270,	16.91820,	17.81720,	18.53890,	19.18790,	19.18790,	19.21330,	19.21330,	19.21330,	19.27780,	19.27780,	19.27780,	19.52620,	19.90790,	19.90790,	19.90790,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.11230,	20.1123,
				2.84764,	5.39857,	7.68025,	9.70261,	11.54610,	13.46480,	15.33530,	16.84530,	17.10250,	18.64980,	19.31300,	19.83140,	19.83140,	19.87300,	19.87300,	19.87300,	19.87300,	19.87300,	19.87300,	19.87300,	19.87300,	19.87300,	19.96900,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.22230,	20.2223,
				2.84028,	5.52519,	7.64074,	9.72899,	11.54500,	13.60860,	15.14470,	16.75830,	17.21750,	18.52000,	19.09780,	19.62040,	19.80720,	20.00920,	20.00920,	20.00920,	20.00920,	20.00920,	20.00920,	20.00920,	20.03150,	20.03150,	20.03150,	20.98970,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.17830,	21.1783,
				2.82285,	5.48680,	7.58555,	9.76152,	11.73100,	13.75060,	15.50410,	16.66540,	17.29900,	18.39570,	19.31650,	19.74270,	19.79470,	20.12800,	20.12800,	20.12800,	20.12800,	20.12800,	20.12800,	20.12800,	20.28340,	20.28340,	20.52860,	20.67450,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.32500,	21.3250,
				2.78201,	5.36975,	7.56839,	9.73333,	11.68520,	13.83650,	15.26700,	16.64580,	17.32490,	18.48900,	19.23560,	19.88540,	20.12320,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.47240,	21.65460,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.93910,	21.9391,
				2.73957,	5.26763,	7.62052,	9.94440,	11.97220,	14.17170,	15.49430,	17.58110,	18.09340,	19.59000,	20.04480,	20.74740,	21.24590,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.69660,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.82940,	23.8294,
				2.76374,	5.50874,	7.86598,	10.12200,	12.39830,	14.76540,	16.26470,	18.31790,	18.85830,	20.68100,	21.28310,	21.92910,	22.75030,	24.65400,	24.65400,	24.65400,	24.65400,	24.65400,	24.97230,	25.41110,	25.41110,	25.41110,	25.83690,	26.17140,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.43060,	26.4306,
				2.73857,	5.53278,	7.82749,	10.11770,	12.45210,	14.67810,	16.34550,	18.31450,	19.41190,	20.56200,	21.83560,	22.37970,	23.21470,	25.33840,	25.33840,	25.33840,	25.38590,	25.51940,	26.29050,	26.70520,	26.79390,	26.79390,	26.93690,	27.21650,	27.58420,	27.58420,	27.58420,	27.86970,	31.29420,	31.29420,	31.29420,	31.29420,	31.29420,	31.29420,	31.29420,	31.29420,	31.29420,	31.29420,	31.29420,	31.2942,
				2.71728,	5.53639,	7.80047,	10.07880,	12.34540,	14.48590,	15.96670,	18.03140,	19.22970,	19.70240,	21.54200,	21.99900,	23.42810,	25.34700,	25.34700,	25.34700,	25.71630,	25.78690,	26.33650,	27.19930,	27.45340,	27.45340,	27.45340,	28.37170,	28.47290,	28.47290,	28.47290,	28.94590,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.33540,	32.3354,
				2.69146,	5.50420,	7.72525,	10.04100,	12.28340,	14.27380,	15.55270,	17.62810,	18.70030,	19.31890,	20.82770,	21.58630,	23.53060,	25.29430,	25.29430,	25.49960,	26.21020,	26.51030,	26.94950,	28.00830,	28.31630,	28.68230,	28.77790,	29.15320,	29.22990,	29.22990,	29.71310,	29.88010,	33.23550,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.58840,	33.5884,
				2.66909,	5.45501,	7.72989,	10.10550,	12.40560,	14.29100,	15.51860,	17.48880,	18.75990,	19.21170,	20.73020,	21.49570,	23.67750,	24.81080,	24.81080,	26.02820,	26.69510,	27.07780,	27.91080,	29.05720,	29.13940,	29.51640,	29.98510,	30.48690,	30.48690,	30.48690,	30.48690,	31.23640,	34.25240,	34.62590,	34.62590,	34.62590,	34.62590,	34.62590,	34.62590,	34.62590,	34.62590,	34.62590,	34.62590,	34.6259,
				2.65678,	5.49741,	7.75469,	10.14040,	12.45360,	14.37580,	15.64190,	17.51650,	18.84550,	19.61840,	20.95810,	21.83170,	24.11320,	25.18430,	25.18430,	27.02270,	27.33340,	27.56140,	28.26660,	29.55880,	29.85280,	30.17720,	30.17720,	31.01730,	31.37020,	31.68550,	31.70850,	32.30740,	34.97740,	35.78150,	35.78150,	35.78150,	35.78150,	35.78150,	35.78150,	35.78150,	35.78150,	35.78150,	35.78150,	35.7815,
				2.64136,	5.42918,	7.70302,	10.16540,	12.41790,	14.31000,	15.91750,	17.44340,	18.87210,	19.69710,	21.06050,	21.87970,	24.16700,	25.06150,	25.38600,	27.40690,	27.92100,	27.92100,	28.47720,	29.70410,	30.06150,	30.78660,	30.78660,	31.47220,	31.60200,	32.01270,	32.12120,	33.02160,	34.85200,	36.08450,	36.26260,	36.26260,	36.26260,	36.26260,	36.26260,	36.26260,	36.26260,	36.26260,	36.26260,	36.2626,
				2.63477,	5.42941,	7.68203,	10.05500,	12.22870,	14.11100,	15.69930,	17.12310,	18.52280,	19.40730,	20.68770,	21.51990,	23.84640,	24.87540,	25.57150,	26.83110,	27.54660,	27.60460,	28.38690,	29.40740,	29.91030,	32.29570,	32.29570,	32.29570,	32.29570,	32.29570,	32.56870,	33.35030,	35.18990,	36.78170,	36.78170,	37.18690,	37.18690,	37.18690,	37.18690,	37.18690,	37.18690,	37.18690,	37.18690,	37.1869,
				2.64571,	5.44961,	7.72084,	10.06230,	12.25070,	14.17540,	15.70700,	17.38740,	18.52660,	19.44550,	20.86460,	21.68430,	23.85570,	25.13110,	25.98030,	26.99030,	27.76010,	27.76010,	28.75390,	29.84770,	30.38340,	32.49150,	32.49150,	32.49150,	32.49150,	32.81810,	33.11800,	34.01160,	35.83660,	36.80480,	36.80480,	38.29280,	38.29280,	38.29280,	38.29280,	38.29280,	38.29280,	38.29280,	38.29280,	38.2928,
				2.63680,	5.41672,	7.75834,	10.05720,	12.19820,	14.07440,	15.58850,	17.29300,	18.42130,	19.35530,	20.66180,	21.59150,	23.58080,	25.32710,	25.83510,	27.10120,	27.94960,	27.94960,	29.04780,	29.63570,	30.53490,	32.43790,	32.43790,	32.43790,	32.43790,	33.19530,	33.19530,	34.08990,	35.64940,	37.36640,	37.64220,	38.03130,	38.03130,	38.03130,	38.03130,	38.03130,	38.03130,	38.03130,	38.03130,	38.0313,
				2.62074,	5.37061,	7.61082,	10.01700,	12.19290,	14.02450,	15.52870,	17.25480,	18.39720,	19.19330,	20.60830,	21.58300,	23.50890,	25.34460,	25.85770,	26.86090,	28.14480,	28.14480,	29.10920,	29.88740,	30.60020,	32.43430,	32.43430,	32.43430,	32.43430,	33.43690,	33.43690,	34.53500,	36.24360,	38.39850,	38.39850,	39.37050,	39.37050,	39.37050,	39.37050,	39.37050,	39.37050,	39.37050,	39.37050,	39.3705,
				2.61108,	5.39319,	7.60319,	9.99883,	12.18450,	14.01890,	15.47240,	17.18100,	18.10880,	19.39710,	21.01110,	21.59570,	23.52440,	25.44270,	26.21700,	26.92730,	28.32550,	28.32550,	29.34320,	30.15100,	31.12170,	31.61990,	31.91330,	33.02770,	33.02770,	33.81010,	33.99720,	35.28800,	37.33840,	39.41970,	39.41970,	40.72520,	40.72520,	40.72520,	40.72520,	40.72520,	40.72520,	40.72520,	40.72520,	40.7252,
				2.61022,	5.37212,	7.59581,	10.01390,	12.19110,	14.05610,	15.43320,	17.16230,	17.92840,	19.23900,	20.85580,	21.42970,	23.49220,	25.19540,	25.75660,	26.73910,	28.95660,	28.95660,	28.95660,	29.70890,	30.67420,	30.88080,	31.74070,	32.48980,	32.86970,	33.33710,	33.76980,	34.76930,	35.05640,	39.29300,	39.29300,	39.93900,	39.93900,	39.93900,	39.93900,	39.93900,	39.93900,	39.93900,	39.93900,	39.9390,
				2.60127,	5.33303,	7.56550,	9.96904,	12.16590,	13.99910,	15.42000,	17.06260,	18.13440,	19.11820,	20.69500,	21.65940,	23.54980,	24.93500,	25.56520,	26.73170,	29.40470,	29.40470,	29.40470,	29.41920,	30.43270,	30.62380,	31.89190,	32.31970,	32.36800,	33.19240,	33.56520,	34.56010,	36.34720,	39.12000,	39.12000,	39.61120,	39.61120,	39.61120,	39.61120,	39.61120,	39.61120,	39.61120,	39.61120,	39.6112,
				2.58574,	5.29717,	7.52973,	9.86195,	12.25220,	14.25110,	15.45160,	17.38340,	18.18370,	19.41680,	20.23110,	21.81970,	24.14220,	25.44570,	25.53990,	27.45650,	28.17970,	28.17970,	28.75550,	31.68510,	31.68510,	31.68510,	32.31810,	33.14210,	33.14210,	34.37450,	34.37450,	34.94050,	37.01220,	39.10010,	39.10010,	40.31340,	40.31340,	40.31340,	40.31340,	40.31340,	40.31340,	40.31340,	40.31340,	40.3134,
				2.59258,	5.19308,	7.56449,	9.93554,	12.31380,	14.33570,	15.60750,	17.57390,	18.25680,	19.65730,	20.47510,	22.14340,	24.51060,	25.89780,	26.05730,	27.71930,	28.61170,	28.61170,	29.30690,	32.27550,	32.27550,	32.27550,	32.90240,	33.84860,	33.84860,	35.15910,	35.15910,	36.18470,	38.08890,	39.84810,	39.84810,	41.07370,	41.07370,	41.07370,	41.07370,	41.07370,	41.07370,	41.07370,	41.07370,	41.0737,
				2.59514,	5.13034,	7.57239,	9.93247,	12.34370,	14.37590,	15.59900,	17.56710,	18.37360,	19.69970,	20.51290,	22.24140,	24.42970,	26.04560,	26.61050,	28.00930,	28.99320,	29.02160,	29.79500,	31.63810,	32.16730,	32.17630,	33.65530,	34.03440,	34.03440,	35.18290,	35.18290,	36.92060,	38.42880,	40.28740,	40.28740,	41.50050,	41.50050,	41.50050,	41.50050,	41.50050,	41.50050,	41.50050,	41.50050,	41.5005,
				2.60043,	5.12232,	7.58589,	9.93001,	12.35110,	14.28090,	15.50890,	17.54300,	18.24720,	19.55470,	20.34020,	22.29820,	24.06720,	25.63080,	26.18510,	27.85570,	28.46720,	29.12780,	29.25130,	30.55590,	31.53010,	31.53010,	33.08920,	33.39570,	33.67080,	34.56150,	34.56150,	36.11270,	37.17770,	39.89760,	39.89760,	40.82670,	40.82670,	40.82670,	40.82670,	40.82670,	40.82670,	40.82670,	40.82670,	40.8267,
				2.57907,	5.09275,	7.52931,	9.86345,	12.18030,	14.22440,	15.29130,	17.36450,	18.14590,	19.48050,	20.25060,	21.69210,	24.05780,	25.43970,	26.19690,	27.71780,	28.38880,	28.73540,	29.24870,	30.79440,	31.23470,	31.23470,	33.17840,	33.70880,	33.78810,	34.71070,	34.71070,	35.69640,	37.87390,	39.61100,	39.61100,	41.11340,	41.11340,	41.11340,	41.11340,	41.11340,	41.11340,	41.11340,	41.11340,	41.1134
			};

			scalability = p_scalability;

			assert(sizeof(p_scalability) / sizeof(p_scalability[0]) == max_cores * number_of_samples);
		}
			break;

		}

		/*
		 * search for well fitting sampling
		 */
		int i = 0;
		for (; i < number_of_samples; i++)
		{
			if (i_workload < scalability_sampling[i])
				break;
		}

		if (i == number_of_samples)
			i = number_of_samples-1;

		*o_max_threads = max_threads[i];
		*o_scalability_graph_size = max_cores;
		*o_scalability_graph = &(scalability[i*max_cores]);

		return i;
	}
};

#endif
